	;
	; Karateka
	;
	; by Oscar Toledo G.
	; (c) Copyright Oscar Toledo G. 1988-2021
	; https://nanochess.org/
	;
	; Creation date: Jun/1988. I was 9 years old.
	; Revision date: May/12/2021. Ported to MSX/Colecovision.
	;

COLECO: EQU 1   ; Define this to 0 for MSX, 1 for Colecovision

RAM_BASE:	EQU $E000-$7000*COLECO
VDP:		EQU $98+$26*COLECO

KEYSEL:	EQU $80
JOYSEL:	EQU $C0
JOY1:	EQU $FC
JOY2:	EQU $FF

    if COLECO
	fname "KARATECV.ROM"

	org $8000,$9fff

	dw $aa55	; No BIOS title screen
	dw 0
	dw 0
	dw 0
	dw 0
	dw START

	jp 0		; RST $08
	jp 0		; RST $10
	jp 0		; RST $18
	jp 0		; RST $20
	jp 0		; RST $28
	jp 0		; RST $30
	jp 0		; RST $38

	jp 0		; No NMI handler

    else
	fname "KARATMSX.ROM"

	org $4000,$5fff

	dw $4241
	dw START
	dw 0
	dw 0
	dw 0
	dw 0
	dw 0
	dw 0

SNSMAT:	equ $0141

    endif

WRTVDP:
	ld a,b
	out (VDP+1),a
	ld a,c
	or $80
	out (VDP+1),a
	ret

SETWRT:
	ld a,l
	out (VDP+1),a
	ld a,h
	or $40
	out (VDP+1),a
	ret

WRTVRM:
	push af
	call SETWRT
	pop af
	out (VDP),a
	ret

FILVRM:
	push af
	call SETWRT
.1:	pop af
	out (VDP),a
	push af
	dec bc
	ld a,b
	or c
	jp nz,.1
	pop af
	ret

	; Setup VDP before game
setup_vdp:
	LD BC,$0200
	CALL WRTVDP
	LD BC,$C201	; No interrupts
	CALL WRTVDP
	LD BC,$0F02	; $3C00 for pattern table
	CALL WRTVDP
	LD BC,$FF03	; $2000 for color table
	CALL WRTVDP
	LD BC,$0304	; $0000 for bitmap table
	CALL WRTVDP
	LD BC,$3605	; $1b00 for sprite attribute table
	CALL WRTVDP
	LD BC,$0706	; $3800 for sprites bitmaps
	CALL WRTVDP
	LD BC,$0407	; Blue border
	CALL WRTVDP
    IF COLECO
	LD HL,($006C)   ; MSX BIOS chars
	LD DE,-128
	ADD HL,DE
    ELSE
	LD HL,($0004)   ; MSX BIOS chars
	INC H
    ENDIF
        PUSH HL
        LD DE,$0100
        LD BC,$0300
        CALL LDIRVM
        POP HL
        PUSH HL
        LD DE,$0900
        LD BC,$0300
        CALL LDIRVM
        POP HL
        LD DE,$1100
        LD BC,$0300
        CALL LDIRVM

        LD HL,$2000
        LD BC,$1800
        LD A,$F4
        CALL FILVRM
	RET

LDIRVM:
        EX DE,HL
.1:     LD A,(DE)
        CALL WRTVRM
        INC DE
        INC HL
        DEC BC
        LD A,B
        OR C
        JR NZ,.1
        RET

GTTRIG:
    if COLECO
        out (KEYSEL),a
        ex (sp),hl
        ex (sp),hl
        in a,(JOY1)
        ld c,a
        in a,(JOY2)
        and c
        ld c,a
	out (JOYSEL),a
	ex (sp),hl
	ex (sp),hl
	in a,(JOY1)
        and c
        ld c,a
	in a,(JOY2)
	and c
	rlca
	rlca
	ccf
	ld a,0
	sbc a,a
	ret
    else
	xor a
	call $00d8
	or a
	ret nz
	ld a,1
	call $00d8
	or a
	ret nz
        ld a,2
        call $00d8
        or a
        ret nz
        ld a,3
        call $00d8
        or a
        ret nz
        ld a,4
        call $00d8
        ret
    endif

	;
	; Gets the joystick direction
	; 0 - No movement
	; 1 - Up
	; 2 - Up + right
	; 3 - Right
	; 4 - Right + down
	; 5 - Down
	; 6 - Down + left
	; 7 - Left
	; 8 - Left + Up
	;
GTSTCK:
    if COLECO
        out (JOYSEL),a
	ex (sp),hl
	ex (sp),hl
        in a,(JOY1)
	ld b,a
	in a,(JOY2)
	and b
        and $0f
        ld c,a
        ld b,0
        ld hl,joy_map
        add hl,bc
        ld a,(hl)
        ret

joy_map:
        db 0,0,0,6,0,0,8,7,0,4,0,5,2,3,1,0

    else
	xor a
	call $00d5
	or a
	ret nz
	ld a,1
	call $00d5
	or a
	ret nz
	ld a,2
	jp $00d5
    endif

	; ROM routines I forgot

	; Clean screen
LIMPIA:	; $04cc
	LD HL,$3C00
	LD BC,$0300
	XOR A
	JP FILVRM

	; Copy string to VDP
INMEDIATO:	; $0169
	EX (SP),HL

.0:	LD A,(HL)
	INC HL
	OR A
	JR Z,.1
	PUSH AF
	POP AF
	OUT (VDP),A
	JR .0

.1:	EX (SP),HL
	RET

	;
	; Start of the game
	;
START:		; 8000
	DI
	LD SP,RAM_BASE+256
JUEGO:
	CALL setup_vdp	; Not in original but needed to setup VDP
	CALL LIMPIA	; Clean screen
	CALL PRESENTACION       ; Title screen
	CALL GRAFICOS	; Setup graphics
PRIN:		; 8009
	CALL PATCH	; Patch for jump
	CALL KARATEKA	; Move karateka
	CALL MALORA	; Move baddie
	CALL NEVAJA	; Move knifes
	JP PRIN		; Repeat game loop

        ; Title screen
PRESENTACION:	; 8018
	LD HL,$3C4D
	CALL SETWRT
	CALL INMEDIATO
	DB "KARATEKA",0

	LD HL,$3ECA
	CALL SETWRT
	CALL INMEDIATO
	DB "PULSE ESPACIO",0

	; Changed from the original
TEC:	XOR A
	CALL GTTRIG
	OR A
	RET NZ
	JP TEC

	; Setup graphics for Karateka, Knife, and Judoka
GRAFICOS:
	LD HL,$3800
	LD DE,GRAFICOS_8600
	LD B,$00
.1:	LD A,(DE)
	CALL WRTVRM
	INC HL
	INC DE
	DJNZ .1

	LD HL,$0B08
	LD DE,GRAFICOS_85D8
	LD B,$28
.2:	LD A,(DE)
	CALL WRTVRM
	INC HL
	INC DE
	DJNZ .2

        LD A,$68
	LD (y_karateka),A
	LD A,$D0
	LD (x_karateka),A
	LD A,$00
	LD (spr_karateka),A
	LD A,$00
	LD (knife1),A
	LD A,$00
	LD (knife2),A
	LD A,$00
        LD (kicks_given),A

	LD HL,$2B00
	LD B,$08
.3:	LD A,$77	; Turquoise color
	CALL WRTVRM
	INC HL
	DJNZ .3
	LD HL,$2808
	LD B,$08
.4:	LD A,$22	; Grass color
	CALL WRTVRM
	INC HL
	DJNZ .4

	LD HL,$2300
	LD B,$08
.5:	LD A,$77
	CALL WRTVRM
	INC HL
	DJNZ .5

	CALL LIMPIA

	LD HL,$3C00
	LD B,$00
.6:	LD A,$60
	CALL WRTVRM
	INC HL
	DJNZ .6

	LD B,$E0
.7:	LD A,$60
	CALL WRTVRM
	INC HL
	DJNZ .7

	LD B,$20
.8:	LD A,$01
	CALL WRTVRM
	INC HL
	DJNZ .8

	LD HL,$2B08
	LD B,$28
.9:	LD A,$17
	CALL WRTVRM
	INC HL
	DJNZ .9

	LD HL,$3DA2
	LD A,$61
	CALL WRTVRM
	LD HL,$3DA3
	LD A,$62
	CALL WRTVRM
	LD HL,$3DC2
	LD A,$63
	CALL WRTVRM
	LD HL,$3DC3
	LD A,$64
	CALL WRTVRM
	LD A,$00
	LD (knife_time+1),A     ; NOT USED
	LD A,$00
	LD (kicks_given),A
	LD A,$00
	LD (knife_time),A
	LD BC,$C201
	CALL WRTVDP
	LD HL,$1B00
	LD A,$68
	CALL WRTVRM
	INC HL
	LD A,$E0
	CALL WRTVRM
	INC HL
	LD A,$00
	CALL WRTVRM
	INC HL
	LD A,$0E
	CALL WRTVRM
	RET

	; Move karateka
KARATEKA:	; 8132
	XOR A
	CALL GTSTCK	; Changed from original
	CP $07
	JP Z,MIK
	CP $03
	JP Z,MDK
	CP $01
	JP Z,GODMA
	CP $05
	JP Z,GODPI
	RET

	; Move karateka to left
MIK:	; 8149
	LD A,(x_karateka)
	SUB $08
	LD (x_karateka),A
	LD HL,$1b01
	CALL WRTVRM
	LD A,(spr_karateka)
	CP $00
	JP Z,SP2
	LD A,$00
	LD (spr_karateka),A
	JP SPR

SP2:	; 8167
	LD A,$08
	LD (spr_karateka),A
SPR:	LD HL,$1b02
	CALL WRTVRM
	RET

	; Move karateka to right
MDK:	; 8173
	LD A,(x_karateka)
	ADD A,$08
	LD (x_karateka),A
	LD HL,$1b01
	CALL WRTVRM
	LD A,(spr_karateka)
	CP $04
	JP Z,SPC
	LD A,$04
	LD (spr_karateka),A
	JP SPR0

SPC:	LD A,$0C
	LD (spr_karateka),A
SPR0:	LD HL,$1b02
	CALL WRTVRM
	RET

	; Hand fight (GOlpe De MAno)
GODMA:	; 819D
	LD A,(spr_karateka)
	CP $00
	JP Z,GAI
	CP $08
	JP Z,GAI
	CP $04
	JP Z,GAD
	CP $0C
	JP Z,GAD
	RET

GAI:	; 81B5
	LD HL,$1B02
	LD A,$10
ZON:	CALL NADEO
	LD BC,$4000
.1:	DEC BC
	LD A,B
	OR C
	JR NZ,.1
	LD A,(spr_karateka)
	JP WRTVRM

GAD:	; 81CB
	LD HL,$1B02
	LD A,$14
	JP ZON

GODPI:	; 81D3 Forgot label on code
	LD A,(spr_karateka)
	CP $00
	JP Z,IZ
	CP $08
	JP Z,IZ
	CP $04
	JP Z,DER
	CP $0C
	JP Z,DER
        RET

IZ:	; 81EB
	LD HL,$1B02
	LD A,$18
HOLA:	CALL NADEO
	LD BC,$4000
BOLAS:	DEC BC
	LD A,B
	OR C
	JR NZ,BOLAS
	LD A,(spr_karateka)
	JP WRTVRM

DER:    ; 8201
	LD HL,$1B02
	LD A,$1C
	JP HOLA

	; Karateka jump
SALTO:	; 8209
	LD A,(y_karateka)
	LD B,$04
OSC:	LD HL,$1B00
	SUB $04
	CALL WRTVRM
	PUSH HL
	PUSH AF
	CALL NEVAJA
	PUSH BC
	LD BC,$4000
.1:	DEC BC
	LD A,B
	OR C
	JR NZ,.1
	POP BC
	POP AF
	POP HL
	DJNZ OSC
	LD B,$04
KAR:	LD HL,$1B00
	ADD A,$04
	CALL WRTVRM
	PUSH HL
	PUSH AF
	CALL NEVAJA
	PUSH BC
	LD BC,$4000
BOCHA:	DEC BC
	LD A,B
	OR C
	JR NZ,BOCHA
	POP BC
	POP AF
	POP HL
	DJNZ KAR
	RET

NEVAJA:	; 8247
	LD A,(knife1)
	ADD A,$A5
	LD H,$3D
	LD L,A
	LD A,$65
	CALL WRTVRM
	LD A,(knife1)
	ADD A,$A4
	LD H,$3D
	LD L,A
	LD A,$60
	CALL WRTVRM
	LD A,(knife1)
	INC A
	LD (knife1),A
	LD A,(knife2)
	ADD A,$C5
	LD H,$3D
	LD L,A
	LD A,$65
	CALL WRTVRM
	LD A,(knife2)
	ADD A,$C4
	LD H,$3D
	LD L,A
	LD A,$60
	CALL WRTVRM
	LD A,(knife2)
	INC A
	LD (knife2),A
	CP $1A
	CALL Z,B2
	LD A,(knife1)
	CP $1A
	CALL Z,B1
	RET

B1:	DEC A
	ADD A,$A5
	LD H,$3D
	LD L,A
	LD A,$60
	CALL WRTVRM
	LD A,$00
	LD (knife1),A
	RET

B2:	DEC A
	ADD A,$C5
	LD H,$3D
	LD L,A
	LD A,$60
	CALL WRTVRM
	LD A,$00
	LD (knife2),A
	RET

MALORA:	; 82B9
	LD A,(x_karateka)
	RRCA
	RRCA
	RRCA
	LD B,A
	LD A,(knife1)
	ADD A,$05
	CP B
	JP Z,MUERTO
	LD A,(knife2)
	ADD A,$05
	CP B
	JP Z,MUERTO
	RET

MUERTO:	; 82D3
	LD HL,$2000
	LD BC,$1800
M2:	LD A,R	
	CALL WRTVRM
	INC HL
	DEC BC
	LD A,B
	OR C
	JR NZ,M2
	LD BC,$0000
M3:	DEC BC
	LD A,B
	OR C
	JR NZ,M3
	JP JUEGO	; Not in original because lack of space

NADEO:	; 82ED
	CALL WRTVRM
	PUSH AF
	PUSH HL
	PUSH BC
	PUSH DE
	LD A,(x_karateka)
	CP $28
	CALL Z,GOLPES
	POP DE
	POP BC
	POP HL
	POP AF
	RET

GOLPES:	; 8301
	LD A,(kicks_given)
	INC A
	LD (kicks_given),A
	CP $14
	JP Z,GANAR
	RET

GANAR:	; 830E
	CALL LIMPIA
	LD HL,$3C03
	CALL SETWRT
	CALL INMEDIATO
	DB "BIEN!",0
	LD BC,$0000
BION:	DEC BC
	LD A,B
	OR C
	JR NZ,BION
	JP JUEGO

PATCH:
	LD BC,$4000
.1:	DEC BC
	LD A,B
	OR C
	JR NZ,.1
	XOR A
	CALL GTTRIG
	OR A
	JP NZ,SALTO
	RET

GRAFICOS_85D8:
	DB $00,$03,$06,$0D,$1B,$17,$03,$01	; JUDOKA
	DB $00,$C0,$00,$C0,$40,$C0,$80,$00
	DB $0F,$F3,$00,$07,$07,$02,$1C,$30
	DB $F0,$CF,$00,$E0,$E0,$40,$38,$0C

	DB $10,$10,$20,$FF,$FF,$20,$10,$10	; KNIFE

GRAFICOS_8600:
	DB $03,$07,$07,$03,$01,$03,$03,$03	; KARATEKA LEFT
	DB $03,$03,$03,$01,$01,$01,$03,$0F
	DB $C0,$E0,$E0,$C0,$80,$C0,$C0,$C0
	DB $C0,$C0,$C0,$80,$80,$80,$80,$80

	DB $03,$07,$07,$03,$01,$03,$03,$03	; KARATEKA RIGHT
	DB $03,$03,$03,$01,$01,$01,$01,$01
	DB $C0,$E0,$E0,$C0,$80,$C0,$C0,$C0
	DB $C0,$C0,$C0,$80,$80,$80,$C0,$F0

	DB $03,$07,$07,$03,$01,$03,$03,$03	; KARATEKA WALKING LEFT
	DB $03,$03,$03,$01,$02,$04,$0C,$3D
	DB $C0,$E0,$E0,$C0,$80,$C0,$C0,$C0
	DB $C0,$C0,$C0,$80,$40,$20,$60,$E0

	DB $03,$07,$07,$03,$01,$03,$03,$03	; KARATEKA WALKING RIGHT
	DB $03,$03,$03,$01,$02,$04,$06,$07
	DB $C0,$E0,$E0,$C0,$80,$C0,$C0,$C0
	DB $C0,$C0,$C0,$80,$40,$20,$30,$BC

	DB $03,$07,$07,$03,$01,$FF,$FF,$03	; KARATEKA PUNCH LEFT
	DB $03,$03,$03,$01,$01,$01,$03,$0F
	DB $C0,$E0,$E0,$C0,$80,$C0,$C0,$C0
	DB $C0,$C0,$C0,$80,$80,$80,$80,$80

	DB $03,$07,$07,$03,$01,$03,$03,$03	; KARATEKA PUNCH RIGHT
	DB $03,$03,$03,$01,$01,$01,$01,$01
	DB $C0,$E0,$E0,$C0,$80,$FF,$FF,$C0
	DB $C0,$C0,$C0,$80,$80,$80,$C0,$F0

	DB $03,$07,$07,$03,$01,$03,$C3,$C3	; KARATEKA KICK LEFT
	DB $33,$33,$0F,$07,$00,$00,$01,$07
	DB $C0,$E0,$E0,$C0,$80,$C0,$C0,$C0
	DB $C0,$C0,$C0,$80,$80,$80,$80,$80

	DB $03,$07,$07,$03,$01,$03,$03,$03	; KARATEKA KICK RIGHT
	DB $03,$03,$03,$01,$01,$01,$01,$01
	DB $C0,$E0,$E0,$C0,$80,$C0,$C3,$C3
	DB $CC,$CC,$F0,$E0,$00,$00,$80,$E0

	ORG RAM_BASE

y_karateka:	RB 1
x_karateka:	RB 1
spr_karateka:	RB 1
knife1:		RB 1
knife2:		RB 1
kicks_given:	RB 1
knife_time:     RB 1    ; NOT USED
